@@ -1,6 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 
    Copyright (C) 2004-2006 Bastien Nocera <hadess@hadess.net>
+   Copyright © 2010 Christian Persch
+   Copyright © 2010 Carlos Garcia Campos
 
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -17,10 +19,11 @@
    write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301  USA.
 
-   Author: Bastien Nocera <hadess@hadess.net>
+   Authors: Bastien Nocera <hadess@hadess.net>
+            Christian Persch
+            Carlos Garcia Campos
  */
 
-
 #include "config.h"
 
 #include <glib/gi18n.h>
@@ -36,32 +39,31 @@
 #endif /* HAVE_XTEST */
 #endif /* GDK_WINDOWING_X11 */
 
-#ifdef WITH_DBUS
-#include <dbus/dbus.h>
-#include <dbus/dbus-glib.h>
+#include "totem-scrsaver.h"
 
 #define GS_SERVICE   "org.gnome.ScreenSaver"
 #define GS_PATH      "/org/gnome/ScreenSaver"
 #define GS_INTERFACE "org.gnome.ScreenSaver"
-#endif /* WITH_DBUS */
-
-#include "totem-scrsaver.h"
 
 #define XSCREENSAVER_MIN_TIMEOUT 60
 
-static GObjectClass *parent_class = NULL;
-static void totem_scrsaver_finalize   (GObject *object);
+enum {
+	PROP_0,
+	PROP_REASON
+};
 
+static void totem_scrsaver_finalize   (GObject *object);
 
 struct TotemScrsaverPrivate {
 	/* Whether the screensaver is disabled */
 	gboolean disabled;
+	/* The reason for the inhibition */
+	char *reason;
 
-#ifdef WITH_DBUS
-	DBusGConnection *connection;
-	DBusGProxy *gs_proxy;
+	GDBusProxy *gs_proxy;
+        gboolean have_screensaver_dbus;
 	guint32 cookie;
-#endif /* WITH_DBUS */
+	gboolean old_dbus_api;
 
 	/* To save the screensaver info */
 	int timeout;
@@ -80,92 +82,128 @@ G_DEFINE_TYPE(TotemScrsaver, totem_scrsaver, G_TYPE_OBJECT)
 static gboolean
 screensaver_is_running_dbus (TotemScrsaver *scr)
 {
-#ifdef WITH_DBUS
-	if (! scr->priv->connection)
-		return FALSE;
+        return scr->priv->have_screensaver_dbus;
+}
 
-	if (! scr->priv->gs_proxy)
-		return FALSE;
+static void
+on_inhibit_cb (GObject      *source_object,
+	       GAsyncResult *res,
+	       gpointer      user_data)
+{
+	GDBusProxy    *proxy = G_DBUS_PROXY (source_object);
+	TotemScrsaver *scr = TOTEM_SCRSAVER (user_data);
+	GVariant      *value;
+	GError        *error = NULL;
 
-	return TRUE;
-#else
-	return FALSE;
-#endif /* WITH_DBUS */
+	value = g_dbus_proxy_call_finish (proxy, res, &error);
+	if (!value) {
+		if (!scr->priv->old_dbus_api &&
+		    g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD)) {
+			g_return_if_fail (scr->priv->reason != NULL);
+			/* try the old API */
+			scr->priv->old_dbus_api = TRUE;
+			g_dbus_proxy_call (proxy,
+					   "InhibitActivation",
+					   g_variant_new ("(s)",
+							  scr->priv->reason),
+					   G_DBUS_CALL_FLAGS_NO_AUTO_START,
+					   -1,
+					   NULL,
+					   on_inhibit_cb,
+					   scr);
+		} else {
+			g_warning ("Problem inhibiting the screensaver: %s", error->message);
+		}
+		g_error_free (error);
+		g_object_unref (scr);
+
+		return;
+	}
+
+	/* save the cookie */
+	if (g_variant_is_of_type (value, G_VARIANT_TYPE ("(u)")))
+		g_variant_get (value, "(u)", &scr->priv->cookie);
+	else
+		scr->priv->cookie = 0;
+	g_variant_unref (value);
+
+	g_object_unref (scr);
+}
+
+static void
+on_uninhibit_cb (GObject      *source_object,
+		 GAsyncResult *res,
+		 gpointer      user_data)
+{
+	GDBusProxy    *proxy = G_DBUS_PROXY (source_object);
+	TotemScrsaver *scr = TOTEM_SCRSAVER (user_data);
+	GVariant      *value;
+	GError        *error = NULL;
+
+	value = g_dbus_proxy_call_finish (proxy, res, &error);
+	if (!value) {
+		if (!scr->priv->old_dbus_api &&
+		    g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD)) {
+			/* try the old API */
+			scr->priv->old_dbus_api = TRUE;
+			g_dbus_proxy_call (proxy,
+					   "AllowActivation",
+					   g_variant_new ("()"),
+					   G_DBUS_CALL_FLAGS_NO_AUTO_START,
+					   -1,
+					   NULL,
+					   on_uninhibit_cb,
+					   scr);
+		} else {
+			g_warning ("Problem uninhibiting the screensaver: %s", error->message);
+		}
+		g_error_free (error);
+		g_object_unref (scr);
+
+		return;
+	}
+
+	/* clear the cookie */
+	scr->priv->cookie = 0;
+	g_variant_unref (value);
+
+	g_object_unref (scr);
 }
 
 static void
 screensaver_inhibit_dbus (TotemScrsaver *scr,
 			  gboolean	 inhibit)
 {
-#ifdef WITH_DBUS
-	GError *error;
-	gboolean res;
+        TotemScrsaverPrivate *priv = scr->priv;
 
-	g_return_if_fail (scr != NULL);
-	g_return_if_fail (scr->priv->connection != NULL);
-	g_return_if_fail (scr->priv->gs_proxy != NULL);
+        if (!priv->have_screensaver_dbus)
+                return;
+
+	scr->priv->old_dbus_api = FALSE;
+	g_object_ref (scr);
 
-	error = NULL;
 	if (inhibit) {
-		char   *application;
-		char   *reason;
-		guint32 cookie;
-
-		application = g_strdup ("Totem");
-		reason = g_strdup (_("Playing a movie"));
-
-		res = dbus_g_proxy_call (scr->priv->gs_proxy,
-					 "Inhibit",
-					 &error,
-					 G_TYPE_STRING, application,
-					 G_TYPE_STRING, reason,
-					 G_TYPE_INVALID,
-					 G_TYPE_UINT, &cookie,
-					 G_TYPE_INVALID);
-
-		if (res) {
-			/* save the cookie */
-			scr->priv->cookie = cookie;
-		} else {
-			/* try the old API */
-			res = dbus_g_proxy_call (scr->priv->gs_proxy,
-						 "InhibitActivation",
-						 &error,
-						 G_TYPE_STRING, reason,
-						 G_TYPE_INVALID,
-						 G_TYPE_INVALID);
-		}
-
-		g_free (reason);
-		g_free (application);
-
+		g_return_if_fail (scr->priv->reason != NULL);
+		g_dbus_proxy_call (priv->gs_proxy,
+				   "Inhibit",
+				   g_variant_new ("(ss)",
+						  g_get_application_name (),
+						  scr->priv->reason),
+				   G_DBUS_CALL_FLAGS_NO_AUTO_START,
+				   -1,
+				   NULL,
+				   on_inhibit_cb,
+				   scr);
 	} else {
-		res = dbus_g_proxy_call (scr->priv->gs_proxy,
-					 "UnInhibit",
-					 &error,
-					 G_TYPE_UINT, scr->priv->cookie,
-					 G_TYPE_INVALID,
-					 G_TYPE_INVALID);
-		if (res) {
-			/* clear the cookie */
-			scr->priv->cookie = 0;
-		} else {
-			/* try the old API */
-			res = dbus_g_proxy_call (scr->priv->gs_proxy,
-						 "AllowActivation",
-						 &error,
-						 G_TYPE_INVALID,
-						 G_TYPE_INVALID);
-		}
+                g_dbus_proxy_call (priv->gs_proxy,
+				   "UnInhibit",
+				   g_variant_new ("(u)", priv->cookie),
+				   G_DBUS_CALL_FLAGS_NO_AUTO_START,
+				   -1,
+				   NULL,
+				   on_uninhibit_cb,
+				   scr);
 	}
-
-	if (! res) {
-		if (error) {
-			g_warning ("Problem inhibiting the screensaver: %s", error->message);
-			g_error_free (error);
-		}
-	}
-#endif /* WITH_DBUS */
 }
 
 static void
@@ -180,59 +218,70 @@ screensaver_disable_dbus (TotemScrsaver *scr)
 	screensaver_inhibit_dbus (scr, TRUE);
 }
 
-#ifdef WITH_DBUS
 static void
-gs_proxy_destroy_cb (GObject *proxy,
-		     TotemScrsaver *scr)
+screensaver_update_dbus_presence (TotemScrsaver *scr)
 {
-	g_warning ("Detected that GNOME screensaver has left the bus");
+        TotemScrsaverPrivate *priv = scr->priv;
+	gchar *name_owner;
 
-	/* just invalidate for now */
-	scr->priv->gs_proxy = NULL;
+	name_owner = g_dbus_proxy_get_name_owner (priv->gs_proxy);
+	if (name_owner) {
+		priv->have_screensaver_dbus = TRUE;
+		g_free (name_owner);
+	} else {
+		priv->have_screensaver_dbus = FALSE;
+	}
+}
+
+static void
+screensaver_dbus_owner_changed_cb (GObject    *object,
+                                   GParamSpec *pspec,
+                                   gpointer    user_data)
+{
+        TotemScrsaver *scr = TOTEM_SCRSAVER (user_data);
+
+	screensaver_update_dbus_presence (scr);
+}
+
+static void
+screensaver_dbus_proxy_new_cb (GObject      *source,
+                               GAsyncResult *result,
+                               gpointer      user_data)
+{
+	TotemScrsaver *scr = TOTEM_SCRSAVER (user_data);
+	TotemScrsaverPrivate *priv = scr->priv;
+
+	priv->gs_proxy = g_dbus_proxy_new_for_bus_finish (result, NULL);
+	if (!priv->gs_proxy)
+		return;
+
+	screensaver_update_dbus_presence (scr);
+
+	g_signal_connect (priv->gs_proxy, "notify::g-name-owner",
+	                  G_CALLBACK (screensaver_dbus_owner_changed_cb),
+	                  scr);
 }
-#endif
 
 static void
 screensaver_init_dbus (TotemScrsaver *scr)
 {
-#ifdef WITH_DBUS
-	GError *error = NULL;
-
-	scr->priv->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-
-	if (! scr->priv->connection) {
-		if (error) {
-			g_warning ("Failed to connect to the session bus: %s", error->message);
-			g_error_free (error);
-		}
-		return;
-	}
-
-	scr->priv->gs_proxy = dbus_g_proxy_new_for_name_owner (scr->priv->connection,
-							       GS_SERVICE,
-							       GS_PATH,
-							       GS_INTERFACE,
-							       NULL);
-	if (scr->priv->gs_proxy != NULL) {
-		g_signal_connect_object (scr->priv->gs_proxy,
-					 "destroy",
-					 G_CALLBACK (gs_proxy_destroy_cb),
-					 scr,
-					 0);
-
-	}
-
-#endif /* WITH_DBUS */
+        g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+	                          G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
+	                          NULL,
+	                          GS_SERVICE,
+	                          GS_PATH,
+	                          GS_INTERFACE,
+	                          NULL,
+	                          screensaver_dbus_proxy_new_cb,
+	                          scr);
 }
 
 static void
 screensaver_finalize_dbus (TotemScrsaver *scr)
 {
-#ifdef WITH_DBUS
 	if (scr->priv->gs_proxy) {
 		g_object_unref (scr->priv->gs_proxy);
 	}
-#endif /* WITH_DBUS */
 }
 
 #ifdef GDK_WINDOWING_X11
@@ -350,16 +399,74 @@ screensaver_finalize_x11 (TotemScrsaver *scr)
 }
 #endif
 
+static void
+totem_scrsaver_get_property (GObject *object,
+			     guint property_id,
+			     GValue *value,
+			     GParamSpec *pspec)
+{
+	TotemScrsaver *scr;
+
+	scr = TOTEM_SCRSAVER (object);
+
+	switch (property_id)
+	{
+	case PROP_REASON:
+		g_value_set_string (value, scr->priv->reason);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+	}
+}
+
+static void
+totem_scrsaver_set_property (GObject *object,
+			     guint property_id,
+			     const GValue *value,
+			     GParamSpec *pspec)
+{
+	TotemScrsaver *scr;
+
+	scr = TOTEM_SCRSAVER (object);
+
+	switch (property_id)
+	{
+	case PROP_REASON:
+		g_free (scr->priv->reason);
+		scr->priv->reason = g_value_dup_string (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+	}
+}
+
 static void
 totem_scrsaver_class_init (TotemScrsaverClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
+	g_type_class_add_private (klass, sizeof (TotemScrsaverPrivate));
 
+	object_class->set_property = totem_scrsaver_set_property;
+	object_class->get_property = totem_scrsaver_get_property;
 	object_class->finalize = totem_scrsaver_finalize;
+
+	g_object_class_install_property (object_class, PROP_REASON,
+					 g_param_spec_string ("reason", NULL, NULL,
+							      NULL, G_PARAM_READWRITE));
+
 }
 
+/**
+ * totem_scrsaver_new:
+ *
+ * Creates a #TotemScrsaver object.
+ * If the GNOME screen saver is running, it uses its DBUS interface to
+ * inhibit the screensaver; otherwise it falls back to using the X
+ * screensaver functionality for this.
+ *
+ * Returns: a newly created #TotemScrsaver
+ */
 TotemScrsaver *
 totem_scrsaver_new (void)
 {
@@ -369,7 +476,9 @@ totem_scrsaver_new (void)
 static void
 totem_scrsaver_init (TotemScrsaver *scr)
 {
-	scr->priv = g_new0 (TotemScrsaverPrivate, 1);
+	scr->priv = G_TYPE_INSTANCE_GET_PRIVATE (scr,
+						 TOTEM_TYPE_SCRSAVER,
+						 TotemScrsaverPrivate);
 
 	screensaver_init_dbus (scr);
 #ifdef GDK_WINDOWING_X11
@@ -412,7 +521,7 @@ totem_scrsaver_enable (TotemScrsaver *scr)
 
 	if (screensaver_is_running_dbus (scr) != FALSE)
 		screensaver_enable_dbus (scr);
-	else 
+	else
 #ifdef GDK_WINDOWING_X11
 		screensaver_enable_x11 (scr);
 #else
@@ -440,6 +549,8 @@ totem_scrsaver_finalize (GObject *object)
 {
 	TotemScrsaver *scr = TOTEM_SCRSAVER (object);
 
+	g_free (scr->priv->reason);
+
 	screensaver_finalize_dbus (scr);
 #ifdef GDK_WINDOWING_X11
 	screensaver_finalize_x11 (scr);
@@ -448,10 +559,5 @@ totem_scrsaver_finalize (GObject *object)
 	{}
 #endif
 
-	g_free (scr->priv);
-
-	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
-		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
-	}
+        G_OBJECT_CLASS (totem_scrsaver_parent_class)->finalize (object);
 }
-
